"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const innerLooper_1 = __importDefault(require("./utils/innerLooper"));
const typeCheckers_1 = require("./utils/typeCheckers");
const constants_1 = require("./utils/constants");
function traversalMap(collection, callbackFn, options) {
    if (options) {
        // If it doesn't throw it means that the options were valid
        validateOptions(options);
    }
    else {
        options = {};
    }
    const SETTINGS = Object.assign({}, constants_1.DEFAULTS, options);
    forEachLoop(collection, callbackFn, '', SETTINGS, Array.isArray(collection), typeCheckers_1.isValidPlainObject(collection));
}
exports.default = traversalMap;
function forEachLoop(value, fn, path, options, valueIsArray, valueIsPlainObject) {
    let loopReturnCode;
    if (valueIsArray || valueIsPlainObject) {
        innerLooper_1.default(value, (childValue, keyOrIndex, parentCollection) => {
            let deepPath;
            if (valueIsArray) {
                deepPath = `${path}[${keyOrIndex}]`;
            }
            else if (valueIsPlainObject) {
                if (options.useDotNotationOnKeys) {
                    deepPath = path ? `${path}.${keyOrIndex}` : keyOrIndex;
                }
                else {
                    deepPath = `${path}['${keyOrIndex}']`;
                }
            }
            const shoudSkipFnCall = options.skipNodes &&
                childValue !== null &&
                typeof childValue === 'object';
            let fnReturnCode;
            if (!shoudSkipFnCall) {
                fnReturnCode = fn.call(parentCollection, childValue, keyOrIndex, deepPath);
            }
            if (fnReturnCode === constants_1.LOOP.BREAK_CURRENT) {
                return false;
            }
            else if (fnReturnCode === constants_1.LOOP.BREAK_ALL) {
                loopReturnCode = fnReturnCode;
                return false;
            }
            /*
             * This is necesary because calling the `fn` may have changed the value.
             * For example if you dont return the value in the `fn` here it will be undefined.
             */
            let childValuePostFn = parentCollection[keyOrIndex];
            let childValuePostFnIsArray = Array.isArray(childValuePostFn);
            let childValuePostFnIsObject = typeCheckers_1.isValidPlainObject(childValuePostFn);
            if (childValuePostFnIsArray || childValuePostFnIsObject) {
                if (fnReturnCode !== constants_1.LOOP.SKIP_CHILDREN) {
                    let childLoopReturnCode = forEachLoop(childValuePostFn, fn, deepPath, options, childValuePostFnIsArray, childValuePostFnIsObject);
                    if (childLoopReturnCode === constants_1.LOOP.BREAK_ALL) {
                        loopReturnCode = childLoopReturnCode;
                        return false;
                    }
                }
            }
        });
    }
    return loopReturnCode;
}
function validateOptions(options) {
    for (let key in options) {
        if (options.hasOwnProperty(key)) {
            const typeOfOption = typeof options[key];
            if (typeOfOption !== 'boolean') {
                throw new TypeError(`Ivalid option, ${key} sould be a boolean, instead got a ${typeOfOption}`);
            }
        }
    }
    return;
}
