"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_isplainobject_1 = __importDefault(require("lodash.isplainobject"));
const es_abstract_is_callable_1 = __importDefault(require("es-abstract-is-callable"));
const es_abstract_has_property_1 = __importDefault(require("es-abstract-has-property"));
/**
 * A utility to mimic the default "foreach" method's means of determining
 * whether or not a property still exists. This is intended to support other
 * "foreach" implementations that don't have this sort of check built in.
 *
 * @param  {Object}  obj - The object to check
 * @param  {String}  P   - The name of the property being checked
 * @return {Boolean}     - A boolean indicating the result of the check
 */
function isPropertyPresent(obj, P) {
    var isPropPresent;
    var objIsArray;
    var objIsArrayLikeObject;
    var objIsPlainObject;
    objIsArray = Array.isArray(obj);
    objIsArrayLikeObject =
        obj != null && !es_abstract_is_callable_1.default(obj) && typeof obj.length === 'number';
    objIsPlainObject = lodash_isplainobject_1.default(obj);
    if ((objIsArray || objIsArrayLikeObject) && isIndex(P, obj.length)) {
        isPropPresent = es_abstract_has_property_1.default;
    }
    else if (objIsPlainObject) {
        isPropPresent = function hasOwnProp(obj, P) {
            return obj.hasOwnProperty(P);
        };
    }
    return isPropPresent(obj, P);
}
function isIndex(value, length) {
    var MAX;
    if (!Number.isFinite(value)) {
        return false;
    }
    MAX = Number.isFinite(length) ? length : Number.MAX_SAFE_INTEGER;
    return value >= 0 && value % 1 === 0 && value <= MAX;
}
exports.default = isPropertyPresent;
