"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typeCheckers_1 = require("./typeCheckers");
function innerLooper(collection, callbackFn, thisArg) {
    if (collection === null) {
        return;
    }
    const collectionIsArray = Array.isArray(collection);
    const collectionIsArrayLikeObject = typeCheckers_1.isArrayLikeObject(collection);
    const collectionIsPlainObject = typeCheckers_1.isValidPlainObject(collection);
    let length;
    const iterable = Object(collection);
    let getPropName = () => { };
    // Setup before iteration
    if (collectionIsArray || collectionIsArrayLikeObject) {
        length = iterable.length;
        getPropName = index => index;
    }
    else if (collectionIsPlainObject) {
        let props = [];
        for (let key in Object(collection)) {
            if (collection.hasOwnProperty(key)) {
                props.push(key);
            }
        }
        length = props.length;
        getPropName = index => props[index];
    }
    let propName;
    let propValue;
    let fnResult;
    let index = 0;
    while (index < length) {
        propName = getPropName(index);
        if (iterable.hasOwnProperty(propName)) {
            propValue = iterable[propName];
            fnResult = callbackFn.call(thisArg, propValue, propName, collection);
            if (fnResult === false) {
                break;
            }
        }
        index++;
    }
}
exports.default = innerLooper;
